#!/usr/bin/python
#
# watch-koji-builds.py - script to watch a set of koji tags for new builds and
#                        kick off tests when new builds/packages are found.
#
# Copyright (c) 2009 Red Hat, Inc.
# Released under the terms of the GPLv2 (or later) license.
# GPLv2+ boilerplate should go here.
# 
# Author: Will Woods <wwoods@redhat.com>

import os
import sys
import koji
import time
import pickle
import xmlrpclib 
import subprocess
import optparse
from autoqa import koji_utils

# Directory where we cache info between runs
cachedir = '/var/cache/autoqa/watch-koji-builds'
cachefile = os.path.join(cachedir,"untagged")
try:
    os.makedirs(cachedir)
except OSError, e:
    if e.errno != 17: # already exists
        raise
# XXX configparser? /etc/koji.conf, section 'koji', item 'server'
#     alternately: read from e.g. /etc/autoqa/autoqa.conf
kojiserver = 'http://koji.fedoraproject.org/kojihub'

# Set of tags to look for
taglist = set(('dist-f10-updates-candidate',
               'dist-f11-updates-candidate',
               'dist-f12-updates-candidate',
               'dist-f13-updates-candidate',
               'dist-f14-updates-candidate',
))
archlist = ('i686', 'x86_64', 'noarch')


def get_prevtime():
    try: 
        return os.stat(cachefile)[8] # mtime
    except:
        return 0

def load_untagged():
    '''Load the list of untagged builds seen by previous runs'''
    try:
        f = open(cachefile)
        untagged = pickle.load(f)
        f.close()
        return untagged
    except:
        return []

def save_list(pkglist):
    '''Save a list of builds for a subsequent run'''
    out = open(cachefile,"w")
    pickle.dump(pkglist, out)
    out.close()

def new_builds_since(session, taglist, prevtime):
    untagged_builds = []
    tagged_builds = {}
    for tag in taglist:
        tagged_builds[tag] = []
    builds = session.list_builds_since(prevtime)
    builds += load_untagged() # untagged builds from previous run
    for nvr in builds:
        tags = set(session.tag_history(nvr))
        if tags:
            # Only check packages with tags from the taglist
            for t in taglist.intersection(tags):
                tagged_builds[t].append(nvr)
        elif not session.build_failed(nvr):
            # No tag yet but the build is still OK - check again later 
            untagged_builds.append(nvr)
    #print "untagged: %s" % " ".join(untagged_builds)
    #print "tagged: %s" % " ".join(tagged_builds)
    return (tagged_builds, untagged_builds)

if __name__ == '__main__':
    # Set up the option parser
    # TODO: optparse for prevtime
    parser = optparse.OptionParser(description='Script to watch a set of koji \
tags for new builds and kick off tests when new builds/packages are found.')
    parser.add_option('--dryrun', '--dry-run', action='store_true',
        help='Do not actually execute commands, just show what would be done')
    parser.add_option('--verbose', action='store_true',
        help='Print extra information')
    (opts, args) = parser.parse_args()

    # load prevtime from some saved timestamp
    prevtime = get_prevtime()
    if not prevtime:
        print "No previous run - checking builds in the past 3 hours"
        prevtime = time.time() - 10800
    if opts.verbose:
        print "Looking up builds since %s" % time.ctime(prevtime)
    # Set up the koji connection
    kojiopts = {} # Possible items: user, password, debug_xmlrpc, debug..
    session = koji_utils.SimpleKojiClientSession(kojiserver, kojiopts)
    untagged_builds = []
    tagged_builds = []
    try:
        session.ensure_connection()
        (tagged_builds, untagged_builds) = new_builds_since(session,
                                            taglist, prevtime)
        if not opts.dryrun:
            save_list(untagged_builds)
        for tag, builds in tagged_builds.items():
            for b in builds:
                # Get a list of all package arches in this build
                arches = [r['arch'] for r in session.listRPMs(b['build_id'])]
                harnesscall = ['autoqa', 'post-koji-build',
                               '--name', b['name'],
                               '--kojitag', tag]
                # Invoke autoqa with an --arch arg for each arch we care about
                testarches = set(arches).intersection(archlist)
                # NOTE HACK: autoqa doesn't handle 'noarch', use 'x86_64'
                if 'noarch' in testarches:
                    testarches.discard('noarch')
                    testarches.add('x86_64')
                for arch in testarches:
                    harnesscall += ['--arch', arch]
                if b['epoch']:
                    envr = '%s:%s' % (b['epoch'], b['nvr'])
                else:
                    envr = b['nvr']
                harnesscall.append(envr)
                if opts.dryrun:
                    print " ".join(harnesscall)
                    continue
                subprocess.call(harnesscall)
    except KeyboardInterrupt:
        print "Exiting on keyboard interrupt."
